using PropertyT

using SCS.SCSSolver
# using Mosek
# using CSDP
# using SDPA

scs_solver(tol, iterations) = SCSSolver(eps=tol, max_iters=iterations, linearsolver=SCS.Direct, alpha=1.95, acceleration_lookback=1)

# solver = Mosek.MosekSolver(
#    MSK_DPAR_INTPNT_CO_TOL_REL_GAP=tol,
#    MSK_IPAR_INTPNT_MAX_ITERATIONS=iterations,
#    QUIET=false)

# solver = CSDP.CSDPSolver(axtol=tol, atytol=tol, objtol=tol, minstepp=tol*10.0^-1, minstepd=tol*10.0^-1)

# solver = SDPA.SDPASolver(epsilonStar=tol, epsilonDash=tol)

include("FPGroups_GAP.jl")

include("groups/Allgroups.jl")
using PropertyTGroups

import PropertyT.Settings

function summarize(sett::PropertyT.Settings)
    info("Threads:     $(Threads.nthreads())")
    info("Workers:     $(workers())")
    info("GroupDir:    $(PropertyT.prepath(sett))")
    info(string(sett.G))
    info("with generating set of size $(length(sett.S))")

    info("Radius:      $(sett.radius)")
    info("Precision:   $(sett.tol)")
    info("Upper bound: $(sett.upper_bound)")
    info("Solver:      $(sett.solver)")
end

function Settings(Gr::PropertyTGroup, args, solver):
    r = get(args, "radius", 2)
    gr_name = PropertyTGroups.name(Gr)*"_r$r"
    G = PropertyTGroups.group(Gr)
    S = PropertyTGroups.generatingset(Gr)

    sol = solver
    ub = get(args,"upper_bound", Inf)
    tol = get(args,"tol", 1e-10)
    ws = get(args, "warmstart", false)

    if get(args, "nosymmetry", false)
        autS = PropertyTGroups.autS(Gr)
        return PropertyT.Settings(gr_name, G, S, r, sol, ub, tol, ws, autS)
    else
        return PropertyT.Settings(gr_name, G, S, r, sol, ub, tol, ws)
    end
end

function main(::PropertyTGroup, sett::Property.Settings)
    isdir(PropertyT.fullpath(sett)) || mkpath(PropertyT.fullpath(sett))

    summarize(sett)

    return PropertyT.check_property_T(sett)
end

function main(::GAPGroup, sett::PropertyT.Settings)
    isdir(PropertyT.fullpath(sett)) || mkpath(PropertyT.fullpath(sett))

    summarize(sett)

    S = [s for s in sett.S if s.symbols[1].pow == 1]
    relations = [k*inv(v) for (k,v) in sett.G.rels]

    prepare_pm_delta(PropertyT.prepath(sett), GAP_groupcode(S, relations), sett.radius)

    return PropertyT.check_property_T(sett)
end
