using ArgParse

###############################################################################
#
#  Parsing command line
#
###############################################################################

function parse_commandline()
    settings = ArgParseSettings()

    @add_arg_table settings begin
        "--tol"
            help = "set numerical tolerance for the SDP solver"
            arg_type = Float64
            default = 1e-6
        "--iterations"
            help = "set maximal number of iterations for the SDP solver"
            arg_type = Int
            default = 50000
        "--upper-bound"
            help = "Set an upper bound for the spectral gap"
            arg_type = Float64
            default = Inf
        "--cpus"
            help = "Set number of cpus used by solver"
            arg_type = Int
            required = false
        "--radius"
            help = "Radius of ball B_r(e,S) to find solution over"
            arg_type = Int
            default = 2
        "--warmstart"
            help = "Use warmstart.jld as the initial guess for SCS"
            action = :store_true
        "--nosymmetry"
            help = "Don't use symmetries of the Laplacian"
            action = :store_true

        "--SL "
            help = "GROUP: the group generated by elementary matrices of size n by n"
            arg_type = Int
            required = false
        "-p"
            help = "Matrices over field of p-elements (p=0 => over ZZ) [only with --SL]"
            arg_type = Int
            default = 0
        "-X"
            help = "Consider EL(N, ZZ⟨X⟩) [only with --SL]"
            action = :store_true

        "--SAut"
            help = "GROUP: the automorphisms group of the free group on N generators"
            arg_type = Int
            required = false

        "--MCG"
            help = "GROUP: mapping class group of surface of genus N"
            arg_type = Int
            required = false

        "--Higman"
            help = "GROUP: the Higman Group"
            action = :store_true

        "--Caprace"
            help = "GROUP: for Caprace Group"
            action = :store_true
    end
    return parse_args(settings)
end

const PARSEDARGS = parse_commandline()

set_parallel_mthread(PARSEDARGS, workers=false)

include("CPUselect.jl")
include("logging.jl")
include("main.jl")


const Gr = PropertyTGroups.PropertyTGroup(PARSEDARGS)
const sett = PropertyT.Settings(PARSEDARGS, Gr,
    solver(args["tol"], args["iterations"]))
isdir(fullpath) || mkpath(fullpath)
logger=setup_logging(PropertyT.filename(fullpath, :fulllog), :fulllog)

main(Gr, sett)
